#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This work was created by participants in the DataONE project, and is
# jointly copyrighted by participating institutions in DataONE. For
# more information on DataONE, see our web site at http://dataone.org.
#
#   Copyright ${year}
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module d1_common.tests.test_pid
===============================

Unit tests for serializaton and de-serialization of the PID type.

:Created: 2011-03-03
:Author: DataONE (Vieglais, Dahl)
:Dependencies:
  - python 2.6
'''

# Stdlib.
import logging
import sys
import unittest
import xml.sax

# 3rd party.
import pyxb

# D1.
from d1_common import xmlrunner
import d1_common.types.generated.dataoneTypes as dataoneTypes

# App
import util


EG_PID_GMN = (
  """<?xml version="1.0" encoding="UTF-8"?>
<d1:identifier xmlns:d1="http://ns.dataone.org/service/types/v1"
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xsi:schemaLocation="http://ns.dataone.org/service/types/v1 file:/home/roger/eclipse_workspace_d1/d1_common_python/src/d1_schemas/dataoneTypes.xsd">testpid</d1:identifier>""",
  'testpid',
)

# TODO.
EG_PID_KNB = (
  """""",
  '',
)


# Blank pid.
EG_BAD_PID_1 = (
  """<?xml version="1.0" encoding="UTF-8"?>
<d1:identifier xmlns:d1="http://ns.dataone.org/service/types/v1"
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xsi:schemaLocation="http://ns.dataone.org/service/types/v1 file:/home/roger/eclipse_workspace_d1/d1_common_python/src/d1_schemas/dataoneTypes.xsd"></d1:identifier>""",
  'testpid',
)

# Missing identifier.                
EG_BAD_PID_2 = (
  """<?xml version="1.0" encoding="UTF-8"?>""",
  'testpid',
)

class TestPID(unittest.TestCase):
  def deserialize_pid_and_check(self, doc, shouldfail=False):
    try:
      obj = dataoneTypes.CreateFromDocument(doc[0])
    except (pyxb.PyXBException, xml.sax.SAXParseException):
      if shouldfail:
        return
      else:
        raise
    if shouldfail:
      raise Exception('Did not receive expected exception')    
    self.assertEqual(obj.value(), doc[1])


  def test_deserialize_gmn(self):
    '''Deserialize: XML -> PID (GMN)'''
    self.deserialize_pid_and_check(EG_PID_GMN)


  def test_deserialize_knb(self):
    '''Deserialize: XML -> PID (KNB)'''
    # TODO.
    #doctest(EG_PID_KNB) 


  def test_deserialize_bad_1(self):
    '''Deserialize: XML -> PID (bad 1)'''
    self.deserialize_pid_and_check(EG_BAD_PID_1, shouldfail=True) 


  def test_deserialize_bad_2(self):
    '''Deserialize: XML -> PID (bad 2)'''
    self.deserialize_pid_and_check(EG_BAD_PID_2, shouldfail=True) 

#===============================================================================
if __name__ == "__main__":
  argv = sys.argv
  if "--debug" in argv:
    logging.basicConfig(level=logging.DEBUG)
    argv.remove("--debug")
  if "--with-xunit" in argv:
    argv.remove("--with-xunit")
    unittest.main(argv=argv, testRunner=xmlrunner.XmlTestRunner(sys.stdout))
  else:
    unittest.main(argv=argv)
